==> trend_etl-1.4.5-py3-none-any.whl: trend_etl/transform/messages.py <==
from pyspark.sql import DataFrame

from pyspark.sql.functions import col, lit, get_json_object, named_struct
from pyspark.sql.types import StringType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    discord_snowflake_timestamp,
    get_created_day,
    parse_messages,
    conv_timestamp,
)


#
# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_messages_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    snowflake_id = discord_snowflake_timestamp(
        get_json_object(col(fields.kafka_value_field).cast(StringType()), "$.id")
    )

    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(snowflake_id).alias(fields.created_day_field),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   guild_idents: guild uuid idents table
#   channel_idents: channel uuid idents table
#   message_idents: message uuid idents table
def from_bronze_get_messages_df(
    streaming_bronze: DataFrame,
    guild_idents: DataFrame,
    channel_idents: DataFrame,
    message_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_messages(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
    )

    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
    )

    result_df = (
        flattened_df.withColumn("created_at", conv_timestamp("timestamp"))
        .drop("timestamp")
        .withColumn("edited_at", conv_timestamp("edited_timestamp"))
        .drop("edited_timestamp")
        .withColumn(
            fields.source_keys_field,
            named_struct(
                lit("id"),
                col("id"),
                lit("channel_id"),
                col("channel_id"),
                lit("message_reference"),
                col("message_reference"),
            ),
        )  # join with channel_idents
        .join(
            channel_idents.withColumnsRenamed(
                {"id": "channel_id", "doc_uuid": "channel_uuid"}
            ),
            "channel_id",
        )
        .drop("channel_id")
        # join with guild_idents
        .join(
            guild_idents.withColumnsRenamed({"id": "guild_id"}),
            "guild_id",
        )
        .drop("guild_id")
        # join with message_idents
        .join(message_idents, "id")
        .drop("id")
        ###
        ### construct reference field from discord's message_reference field
        ###
        # join to get reference for: guild_id
        .join(
            guild_idents.withColumnsRenamed(
                {"group_id": "__group_id", "id": "guild_id"}
            ),
            col("message_reference.guild_id") == col("guild_id"),
            "left_outer",
        )
        # join to get reference for: channel_id
        .join(
            channel_idents.withColumnsRenamed(
                {"doc_uuid": "__channel_uuid", "id": "channel_id"}
            ),
            col("message_reference.channel_id") == col("channel_id"),
            "left_outer",
        )
        # join to get reference for: message_id
        .join(
            message_idents.withColumnsRenamed(
                {"doc_uuid": "__message_uuid", "id": "message_id"}
            ),
            col("message_reference.message_id") == col("message_id"),
            "left_outer",
        )
        .withColumn(
            "reference",
            named_struct(
                lit("type"),
                col("message_reference.type"),
                lit("group_id"),
                col("__group_id"),
                lit("channel_uuid"),
                col("__channel_uuid"),
                lit("message_uuid"),
                col("__message_uuid"),
            ),
        )
        .drop(
            "__group_id",
            "guild_id",
            "__channel_uuid",
            "channel_id",
            "__message_uuid",
            "message_id",
            "message_reference",
        )
    )

    return result_df

==> trend_etl-1.4.14-py3-none-any.whl: trend_etl/transform/messages.py <==
from pyspark.sql import DataFrame

from pyspark.sql.functions import col, lit, get_json_object, struct, exists
from pyspark.sql.types import StringType

import trend_etl.schema.fields as fields
from trend_etl.functions.misc import get_created_day
from trend_etl.functions.udfs import conv_timestamp, discord_snowflake_timestamp
from trend_etl.functions.parsing import parse_messages


#
# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_messages_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    snowflake_id = discord_snowflake_timestamp(
        get_json_object(col(fields.kafka_value_field).cast(StringType()), "$.id")
    )

    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(snowflake_id).alias(fields.created_day_field),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   guild_idents: guild uuid idents table
#   channel_idents: channel uuid idents table
#   message_idents: message uuid idents table
def from_bronze_get_messages_df(
    streaming_bronze: DataFrame,
    guild_idents: DataFrame,
    channel_idents: DataFrame,
    message_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_messages(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
    )

    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
    )

    bot_author = col("author.bot").isNotNull() & col("author.bot")
    mentions_bot = exists(
        "mentions", lambda mention: mention.bot.isNotNull() & mention.bot
    )
    filter_bot_messages = flattened_df.where(~bot_author & ~mentions_bot)

    result_df = (
        filter_bot_messages.withColumn("created_at", conv_timestamp("timestamp"))
        .drop("timestamp")
        .withColumn("edited_at", conv_timestamp("edited_timestamp"))
        .drop("edited_timestamp")
        .withColumn(
            fields.source_keys_field,
            struct(
                col("id").alias("id"),
                col("channel_id").alias("channel_id"),
                col("message_reference").alias("message_reference"),
            ),
        )  # join with channel_idents
        .join(
            channel_idents.withColumnsRenamed(
                {"id": "channel_id", "doc_uuid": "channel_uuid"}
            ),
            "channel_id",
        )
        .drop("channel_id")
        # join with guild_idents
        .join(
            guild_idents.withColumnsRenamed({"id": "guild_id"}),
            "guild_id",
        )
        .drop("guild_id")
        # join with message_idents
        .join(message_idents, "id")
        .drop("id")
        ###
        ### construct reference field from discord's message_reference field
        ###
        # join to get reference for: guild_id
        .join(
            guild_idents.withColumnsRenamed(
                {"group_id": "__group_id", "id": "guild_id"}
            ),
            col("message_reference.guild_id") == col("guild_id"),
            "left_outer",
        )
        # join to get reference for: channel_id
        .join(
            channel_idents.withColumnsRenamed(
                {"doc_uuid": "__channel_uuid", "id": "channel_id"}
            ),
            col("message_reference.channel_id") == col("channel_id"),
            "left_outer",
        )
        # join to get reference for: message_id
        .join(
            message_idents.withColumnsRenamed(
                {"doc_uuid": "__message_uuid", "id": "message_id"}
            ),
            col("message_reference.message_id") == col("message_id"),
            "left_outer",
        )
        .withColumn(
            "reference",
            struct(
                col("message_reference.type").alias("type"),
                col("__group_id").alias("group_id"),
                col("__channel_uuid").alias("channel_uuid"),
                col("__message_uuid").alias("message_uuid"),
            ),
        )
        .drop(
            "__group_id",
            "guild_id",
            "__channel_uuid",
            "channel_id",
            "__message_uuid",
            "message_id",
            "message_reference",
        )
    )

    return result_df
